!***********************************************************************
! Perform binary collisions using Monte Carlo method proposed by Nanbu
! and Yonemura, JCP, 145, 639, 1998 and K. Nanbu, Phys. Rev., E55, 4642,
! 1997). This subroutine processes some housekeeping work for the
! collision algorithm.
!
! Jiannan TU 5/22/2010
!***********************************************************************
subroutine coulomb_coll() !(energy)
    use gskparm

    integer :: indO,indH,indHe,i,j,k
    real*8, dimension(np,3) :: dummyO,dummyH,dummyHe
    real*8 :: s,ddt,Te,ne,TiO,TiH,TiHe,niO,niH,niHe,viO,viH,viHe
    real*8 :: ran2,LnA,LnAab !,energy
    real*8, dimension(3) :: temp

    integer rank,ierr,status
    common /mpi/rank,ierr,status


    indO=imax_O; indH=imax_H; indHe=imax_He

        !randomize ptcl arrays
        do j=1,indO
            k=int((indO-j)*ran2(iseed)+j)
            temp(1:3)=Oxygen(j,1:3)
            Oxygen(j,1:3)=Oxygen(k,1:3)
            Oxygen(k,1:3)=temp(1:3)
        enddo

        do j=1,indH
            k=int((indH-j)*ran2(iseed)+j)
            temp(1:3)=Hydrog(j,1:3)
            Hydrog(j,1:3)=Hydrog(k,1:3)
            Hydrog(k,1:3)=temp(1:3)
        enddo

        do j=1,indHe
            k=int((indHe-j)*ran2(iseed)+j)
            temp(1:3)=Helium(j,1:3)
            Helium(j,1:3)=Helium(k,1:3)
            Helium(k,1:3)=temp(1:3)
        enddo

        !ne, Te, TO & TH are the average of those at i and i+1
        ne=den_e(rank+1)
        Te=(tpr_e(rank+1)+2.0d0*tpp_e(rank+1))/3.0d0
        TiO=(tpr_O(rank+1)+2.0d0*tpp_O(rank+1))/3.0d0
        TiH=(tpr_H(rank+1)+2.0d0*tpp_H(rank+1))/3.0d0
        TiHe=(tpr_He(rank+1)+2.0d0*tpp_He(rank+1))/3.0d0
        niO=den_O(rank+1)
        niH=den_H(rank+1)
        niHe=den_He(rank+1)
        viO=vel_O(rank+1)
        viH=vel_H(rank+1)
        viHe=vel_He(rank+1)


        ! Coulomb logarithm
        LnA=LnAab(1,2,Te,ne,TiO,TiH,viO,viH)

        !O+ - H+ collisions. 
        if(indO.ge.indH) then
            ddt=dtg*max(rep_O,rep_H)/rep_H  ! dtg-time increment per real particle
            call mutual_coll(Oxygen,Hydrog,indO,indH,1,2,ddt,niH,rep_O,rep_H,LnA)
        else
            ddt=dtg*max(rep_H,rep_O)/rep_O
            call mutual_coll(Hydrog,Oxygen,indH,indO,2,1,ddt,niO,rep_H,rep_O,LnA)
        endif

        !H+ - He+ collisions. 
        LnA=LnAab(2,3,Te,ne,TiH,TiHe,viH,viHe)

        if(indH.ge.indHe) then
            ddt=dtg*max(rep_H,rep_He)/rep_He
            call mutual_coll(Hydrog,Helium,indH,indHe,2,3,ddt,niHe,rep_H,rep_He,LnA)
        else
            ddt=dtg*max(rep_He,rep_H)/rep_H
            call mutual_coll(Helium,Hydrog,indHe,indH,3,2,ddt,niH,rep_He,rep_H,LnA)
        endif


        !He+ - O+ collisions. 
        LnA=LnAab(3,1,Te,ne,TiHe,TiO,viHe,viO)

        if(indHe.ge.indO) then
            ddt=dtg*max(rep_He,rep_O)/rep_O
            call mutual_coll(Helium,Oxygen,indHe,indO,3,1,ddt,niO,rep_He,rep_O,LnA)
        else
            ddt=dtg*max(rep_O,rep_He)/rep_He
            call mutual_coll(Oxygen,Helium,indO,indHe,1,3,ddt,niHe,rep_O,rep_He,LnA)
        endif

        !O+ - O+ self collisions
        LnA=LnAab(1,1,Te,ne,TiO,TiO,viO,viO)

        ddt=dtg
        if(mod(indO,2).ne.0) ddt=(dfloat(indO)/dfloat(indO+1))*dtg
        call self_coll(Oxygen,indO,1,ddt,niO,LnA)

        !H+ - H+ self collisions
        LnA=LnAab(2,2,Te,ne,TiH,TiH,viH,viH)

        ddt=dtg
        if(mod(indH,2).ne.0) ddt=(dfloat(indH)/dfloat(indH+1))*dtg
        call self_coll(Hydrog,indH,2,ddt,niH,LnA)

        !He+ - He+ self collisions
        LnA=LnAab(3,3,Te,ne,TiHe,TiHe,viHe,viHe)

        ddt=dtg
        if(mod(indHe,2).ne.0) ddt=(dfloat(indHe)/dfloat(indHe+1))*dtg
        call self_coll(Helium,indHe,3,ddt,niHe,LnA)


end subroutine
!***********************************************************************
! Collisions between ions of different species
!
! Jiannan TU 5/22/2010
!***********************************************************************
subroutine mutual_coll(dummya,dummyb,inda,indb,a,b,dt,nib,repa,repb,LnA)
    use gskparm

    integer :: inda,indb,a,b,j,k
    real*8, dimension(np,3) :: dummya
    real*8, dimension(np,3) :: dummyb
    real*8 :: dt,nib,repa,repb,LnA
    real*8 :: Zaa,Zbb,proba,probb,ran2,vp,pi2,phi
    real*8, dimension(3) :: v1,v2,v1sav,v2sav

    data pi2/6.283185307d0/

    proba=repb/max(repa,repb)
    probb=repa/max(repa,repb)

    do j=1,inda
        v1(1)=dummya(j,2)
        vp=dummya(j,3)
        phi=pi2*ran2(iseed)
        v1(2)=vp*cos(phi)
        v1(3)=vp*sin(phi)

        k=indb*ran2(iseed)+1
        v2(1)=dummyb(k,2)
        vp=dummyb(k,3)
        phi=pi2*ran2(iseed)
        v2(2)=vp*cos(phi)
        v2(3)=vp*sin(phi)

        
        v1sav=v1; v2sav=v2
        call coll_vel_chng(a,b,dt,nib,LnA,v1,v2,iseed)
        if(ran2(iseed).le.proba) then
            Zaa=1.0d0
        else
            Zaa=0.0d0
        endif

        if(ran2(iseed).le.probb) then
            Zbb=1.0d0
        else
            Zbb=0.0d0
        endif

        v1=(1.0d0-Zaa)*v1sav+Zaa*v1
        v2=(1.0d0-Zbb)*v2sav+Zbb*v2

        dummya(j,2)=v1(1)
        dummya(j,3)=sqrt(v1(2)**2+v1(3)**2)
        dummyb(k,2)=v2(1)
        dummyb(k,3)=sqrt(v2(2)**2+v2(3)**2)
    enddo

end subroutine
!***********************************************************************
! Collisions between ions of the same speices
!
! Jiannan TU 5/22/2010
!***********************************************************************
subroutine self_coll(dummy,ind,a,dt,ni,LnA)
    use gskparm

    integer :: ind,a,j,k,indhf
    real*8, dimension(np,3) :: dummy
    real*8 :: dt,ni,LnA,vp,pi2,phi,ran2
    real*8, dimension(3) :: v1,v2

    data pi2/6.283185307d0/

    indhf=ind/2
    do j=1,indhf
        v1(1)=dummy(j,2)
        vp=dummy(j,3)
        phi=pi2*ran2(iseed)
        v1(2)=vp*cos(phi)
        v1(3)=vp*sin(phi)

        k=indhf+j
        v2(1)=dummy(k,2)
        vp=dummy(k,3)
        phi=pi2*ran2(iseed)
        v2(2)=vp*cos(phi)
        v2(3)=vp*sin(phi)

        call coll_vel_chng(a,a,dt,ni,LnA,v1,v2,iseed)

        dummy(j,2)=v1(1)
        dummy(j,3)=sqrt(v1(2)**2+v1(3)**2)
        dummy(k,2)=v2(1)
        dummy(k,3)=sqrt(v2(2)**2+v2(3)**2)
    enddo

    if(mod(ind,2).ne.0) then
        v1(1)=dummy(ind,2)
        vp=dummy(ind,3)
        phi=pi2*ran2(iseed)
        v1(2)=vp*cos(phi)
        v1(3)=vp*sin(phi)

        v2(1)=dummy(1,2)
        vp=dummy(1,3)
        phi=pi2*ran2(iseed)
        v2(2)=vp*cos(phi)
        v2(3)=vp*sin(phi)

        call coll_vel_chng(a,a,dt,ni,LnA,v1,v2,iseed)

        dummy(ind,2)=v1(1)
        dummy(ind,3)=sqrt(v1(2)**2+v1(3)**2)
        dummy(1,2)=v2(1)
        dummy(1,3)=sqrt(v2(2)**2+v2(3)**2)
    endif
end subroutine
!***********************************************************************
! Calculate postcollision velocity (see Nanbu and Yonemura, JCP, 145, 
! 639, 1998 and K. Nanbu, Phys. Rev., E55, 4642, 1997)
!
! Jiannan TU 5/20/2010
!***********************************************************************
subroutine coll_vel_chng(a,b,dt,nb,LnA,va,vb,iseed)
    integer :: a,b,iseed,i
    real*8 :: dt,nb,LnA
    real*8, dimension(3) :: va,vb,hh,gg
    real*8 :: gab,gp,pi2,gh !,mba,mab

    real*8 :: acosx,sinx,cosep,sinep,cosx,epsilon,ran2
    real*8, dimension(3,3) :: mc
    !real*8, dimension(3) :: mc

    data pi2/6.283185307d0/

! (a,b)=(1,1): O+ - O+; (a,b)=(1,2): O+ - H+;  (a,b)=(1,3): O+ - He+; 
! (a,b)=(2,2): H+ - H+; (a,b)=(2,3): H+ - He+; (a,b)=(3,3): He+ - He+;
! mc(1,1)=1/2; mc(2,1)=mH/(mO+mH);  mc(3,1)=mHe/(mO+MHe)
! mc(1,2)=mO/(mH+mO);  mc(2,2)=1/2; mc(3,2)=mHe/(mH+mHe)
! mc(1,3)=mO/(mHe+mO); mc(2,3)=mH/(mHe+mH); mc(3,3)=1/2
    data mc/0.5d0,0.058823529d0,0.2d0,0.94117647d0,0.5d0,0.8d0,0.8d0,0.2d0,0.5d0/
!    data mc/0.5d0,0.8333333d0,0.2d0,0.16666666d0,0.5d0,0.8d0,0.8d0,0.2d00,0.5d0/

    gg(1)=va(1)-vb(1)
    gg(2)=va(2)-vb(2)
    gg(3)=va(3)-vb(3)

    gp=sqrt(gg(2)**2+gg(3)**2)
    gab=sqrt(gg(1)**2+gp**2)

    acosx=cosx(a,b,dt,nb,gab,LnA,iseed)
    sinx=sqrt(1.d0-acosx**2)
    acosx=1.d0-acosx

    epsilon=pi2*ran2(iseed)

    cosep=cos(epsilon)
    sinep=sin(epsilon)

    hh(1)=gp*cosep
    hh(2)=-(gg(2)*gg(1)*cosep+gab*gg(3)*sinep)/gp
    hh(3)=-(gg(3)*gg(1)*cosep-gab*gg(2)*sinep)/gp

    do i=1,3
        gh=gg(i)*acosx+hh(i)*sinx
        va(i)=va(i)-mc(b,a)*gh
        vb(i)=vb(i)+mc(a,b)*gh
    enddo

end subroutine
!***********************************************************************
! cosx (Eq(5) in Nanbu and Yonemura, JCP, 145, 639, 1998. See also 
! K. Nanbu, Phys. Rev., E55, 4642, 1997)
!
! Jiannan TU 5/19/2010
!***********************************************************************
real*8 function cosx(a,b,dt,nb,gab,LnA,iseed)
    integer :: a,b,iseed
    real*8 :: dt,nb,gab,LnA
    real*8 :: Aab,s,sab,ran2
    real*8, dimension(400) :: ss,AA

    common /colltable/ss,AA

! Sab (eq(4) of Nanbu and Yonemura, JCP, 145, 639, 1998)
    s=sab(a,b,LnA,dt,nb,gab)

! Calculate parameter A (eq(3))
    if(s.lt.0.01d0) then
        Aab=1.d0/s
    else if(s.ge.0.01d0.and.s.le.4.0d0) then
        !interpolate using pre-tabulated values
        call lgrg(ss,AA,400,s,Aab)
    else if(s.gt.4.0d0.and.s.le.80.0d0) then
        Aab=3.0*exp(-s)
    else
        Aab=5.4d-35
    endif

! Now calculated cosX (eq(5))
    if(Aab.gt.106.d0) then
        cosx=1.d0+s*log(ran2(iseed))
    else if(Aab.lt.1.0d-3) then
        cosx=2.0d0*ran2(iseed)-1.0d0
    else
        cosx=log(exp(-Aab)+2.d0*ran2(iseed)*sinh(Aab))/Aab
    endif
end function

!***********************************************************************
! function Sab (Eq(4) in Nanbu and Yonemura, JCP, 145, 639, 1998)
!
! Jiannan TU 5/19/2010
!***********************************************************************
real*8 function sab(a,b,LnA,dt,nb,gab)
    integer :: a,b
    real*8 :: LnA,dt,nb,gab
    real*8,dimension(3,3) :: cab

! in order: O+ - O+; O+ - H+; O+ - He+; (a,b): (1,1); (1,2); (1,3)
!           H+ - O+; H+ - H+; H+ - He+;        (2,1); (2,2); (2,3)
!           He+ -O+; He+ -H+; He+ -He+         (3,1); (3,2); (3,3)
    data cab/1.442432895d-17,1.042157767d-15,9.015205595d-17, &
             1.042157767d-15,3.692628212d-15,1.442432895d-15, &
             9.015205595d-17,1.442432895d-15,2.307892632d-16/

    sab=cab(a,b)*LnA*nb*dt/gab**3
end function

!***********************************************************************
! Coulomb logarithm (Gven in Nanbu and Yonemura, JCP, 145, 639, 1998)
!
! Jiannan TU 5/19/2010
!***********************************************************************
real*8 function LnAab(a,b,Te,ne,Ta,Tb,vva,vvb)
    integer:: a,b
    real*8 :: Te,ne,Ta,Tb,vva,vvb,gav2

    real*8, dimension(3,3) :: cab
    real*8, dimension(3) :: kbm

! in order: O+ - O+; O+ - H+; O+ - He+; (a,b): (1,1); (1,2); (1,3)
!           H+ - O+; H+ - H+; H+ - He+;        (2,1); (2,2); (2,3)
!           He+ -O+; He+ -H+; He+ -He+         (3,1); (3,2); (3,3)
! I calculate including transfering unit of ne into cm-3
    data cab/0.690010011d0,-1.450056153d0,-0.226280721d0,&
             -1.450056153d0,-2.082578711d0,-1.612575082d0,&
             -0.226280721d0,-1.612575082d0,-0.696284350d0/

! kbm=3kb/m (O+; H+; He+)
    data kbm/1547.777412d0,24764.4386d0,6191.10965d0/

    gav2=kbm(a)*Ta+kbm(b)*Tb+(vva-vvb)**2

    LnAab=cab(a,b)+0.5d0*log(Te/ne*gav2)
end function

!***********************************************************************
! tabulated values of A for Eq(3) in Nanbu and Yonemura, JCP, 145, 639,
! 1998)
!
! Jiannan TU 5/19/2010
!***********************************************************************
BLOCK DATA ssAA
    real*8, dimension(400) :: ss,AA
    common /colltable/ss,AA

    data ss/0.01d0, 0.02d0, 0.03d0, 0.04d0, 0.05d0, 0.06d0, 0.07d0, 0.08d0, 0.09d0, 0.10d0, &
0.11d0, 0.12d0, 0.13d0, 0.14d0, 0.15d0, 0.16d0, 0.17d0, 0.18d0, 0.19d0, 0.20d0, &
0.21d0, 0.22d0, 0.23d0, 0.24d0, 0.25d0, 0.26d0, 0.27d0, 0.28d0, 0.29d0, 0.30d0, &
0.31d0, 0.32d0, 0.33d0, 0.34d0, 0.35d0, 0.36d0, 0.37d0, 0.38d0, 0.39d0, 0.40d0, &
0.41d0, 0.42d0, 0.43d0, 0.44d0, 0.45d0, 0.46d0, 0.47d0, 0.48d0, 0.49d0, 0.50d0, &
0.51d0, 0.52d0, 0.53d0, 0.54d0, 0.55d0, 0.56d0, 0.57d0, 0.58d0, 0.59d0, 0.60d0, &
0.61d0, 0.62d0, 0.63d0, 0.64d0, 0.65d0, 0.66d0, 0.67d0, 0.68d0, 0.69d0, 0.70d0, &
0.71d0, 0.72d0, 0.73d0, 0.74d0, 0.75d0, 0.76d0, 0.77d0, 0.78d0, 0.79d0, 0.80d0, &
0.81d0, 0.82d0, 0.83d0, 0.84d0, 0.85d0, 0.86d0, 0.87d0, 0.88d0, 0.89d0, 0.90d0, &
0.91d0, 0.92d0, 0.93d0, 0.94d0, 0.95d0, 0.96d0, 0.97d0, 0.98d0, 0.99d0, 1.00d0, &
1.01d0, 1.02d0, 1.03d0, 1.04d0, 1.05d0, 1.06d0, 1.07d0, 1.08d0, 1.09d0, 1.10d0, &
1.11d0, 1.12d0, 1.13d0, 1.14d0, 1.15d0, 1.16d0, 1.17d0, 1.18d0, 1.19d0, 1.20d0, &
1.21d0, 1.22d0, 1.23d0, 1.24d0, 1.25d0, 1.26d0, 1.27d0, 1.28d0, 1.29d0, 1.30d0, &
1.31d0, 1.32d0, 1.33d0, 1.34d0, 1.35d0, 1.36d0, 1.37d0, 1.38d0, 1.39d0, 1.40d0, &
1.41d0, 1.42d0, 1.43d0, 1.44d0, 1.45d0, 1.46d0, 1.47d0, 1.48d0, 1.49d0, 1.50d0, &
1.51d0, 1.52d0, 1.53d0, 1.54d0, 1.55d0, 1.56d0, 1.57d0, 1.58d0, 1.59d0, 1.60d0, &
1.61d0, 1.62d0, 1.63d0, 1.64d0, 1.65d0, 1.66d0, 1.67d0, 1.68d0, 1.69d0, 1.70d0, &
1.71d0, 1.72d0, 1.73d0, 1.74d0, 1.75d0, 1.76d0, 1.77d0, 1.78d0, 1.79d0, 1.80d0, &
1.81d0, 1.82d0, 1.83d0, 1.84d0, 1.85d0, 1.86d0, 1.87d0, 1.88d0, 1.89d0, 1.90d0, &
1.91d0, 1.92d0, 1.93d0, 1.94d0, 1.95d0, 1.96d0, 1.97d0, 1.98d0, 1.99d0, 2.00d0, &
2.01d0, 2.02d0, 2.03d0, 2.04d0, 2.05d0, 2.06d0, 2.07d0, 2.08d0, 2.09d0, 2.10d0, &
2.11d0, 2.12d0, 2.13d0, 2.14d0, 2.15d0, 2.16d0, 2.17d0, 2.18d0, 2.19d0, 2.20d0, &
2.21d0, 2.22d0, 2.23d0, 2.24d0, 2.25d0, 2.26d0, 2.27d0, 2.28d0, 2.29d0, 2.30d0, &
2.31d0, 2.32d0, 2.33d0, 2.34d0, 2.35d0, 2.36d0, 2.37d0, 2.38d0, 2.39d0, 2.40d0, &
2.41d0, 2.42d0, 2.43d0, 2.44d0, 2.45d0, 2.46d0, 2.47d0, 2.48d0, 2.49d0, 2.50d0, &
2.51d0, 2.52d0, 2.53d0, 2.54d0, 2.55d0, 2.56d0, 2.57d0, 2.58d0, 2.59d0, 2.60d0, &
2.61d0, 2.62d0, 2.63d0, 2.64d0, 2.65d0, 2.66d0, 2.67d0, 2.68d0, 2.69d0, 2.70d0, &
2.71d0, 2.72d0, 2.73d0, 2.74d0, 2.75d0, 2.76d0, 2.77d0, 2.78d0, 2.79d0, 2.80d0, &
2.81d0, 2.82d0, 2.83d0, 2.84d0, 2.85d0, 2.86d0, 2.87d0, 2.88d0, 2.89d0, 2.90d0, &
2.91d0, 2.92d0, 2.93d0, 2.94d0, 2.95d0, 2.96d0, 2.97d0, 2.98d0, 2.99d0, 3.00d0, &
3.01d0, 3.02d0, 3.03d0, 3.04d0, 3.05d0, 3.06d0, 3.07d0, 3.08d0, 3.09d0, 3.10d0, &
3.11d0, 3.12d0, 3.13d0, 3.14d0, 3.15d0, 3.16d0, 3.17d0, 3.18d0, 3.19d0, 3.20d0, &
3.21d0, 3.22d0, 3.23d0, 3.24d0, 3.25d0, 3.26d0, 3.27d0, 3.28d0, 3.29d0, 3.30d0, &
3.31d0, 3.32d0, 3.33d0, 3.34d0, 3.35d0, 3.36d0, 3.37d0, 3.38d0, 3.39d0, 3.40d0, &
3.41d0, 3.42d0, 3.43d0, 3.44d0, 3.45d0, 3.46d0, 3.47d0, 3.48d0, 3.49d0, 3.50d0, &
3.51d0, 3.52d0, 3.53d0, 3.54d0, 3.55d0, 3.56d0, 3.57d0, 3.58d0, 3.59d0, 3.60d0, &
3.61d0, 3.62d0, 3.63d0, 3.64d0, 3.65d0, 3.66d0, 3.67d0, 3.68d0, 3.69d0, 3.70d0, &
3.71d0, 3.72d0, 3.73d0, 3.74d0, 3.75d0, 3.76d0, 3.77d0, 3.78d0, 3.79d0, 3.80d0, &
3.81d0, 3.82d0, 3.83d0, 3.84d0, 3.85d0, 3.86d0, 3.87d0, 3.88d0, 3.89d0, 3.90d0, &
3.91d0, 3.92d0, 3.93d0, 3.94d0, 3.95d0, 3.96d0, 3.97d0, 3.98d0, 3.99d0, 4.00d0/

    data AA/0.1005008D+03, 0.5050167D+02, 0.3383583D+02, 0.2550333D+02, 0.2050417D+02, &
0.1717167D+02, 0.1479155D+02, 0.1300667D+02, 0.1161861D+02, 0.1050833D+02, &
0.9600073D+01, 0.8843328D+01, 0.8203128D+01, 0.7654494D+01, 0.7179102D+01, &
0.6763205D+01, 0.6396285D+01, 0.6070154D+01, 0.5778342D+01, 0.5515671D+01, &
0.5277941D+01, 0.5061716D+01, 0.4864155D+01, 0.4682890D+01, 0.4515930D+01, &
0.4361594D+01, 0.4218447D+01, 0.4085265D+01, 0.3960991D+01, 0.3844712D+01, &
0.3735635D+01, 0.3633067D+01, 0.3536400D+01, 0.3445102D+01, 0.3358701D+01, &
0.3276778D+01, 0.3198964D+01, 0.3124926D+01, 0.3054370D+01, 0.2987030D+01, &
0.2922667D+01, 0.2861066D+01, 0.2802032D+01, 0.2745390D+01, 0.2690979D+01, &
0.2638654D+01, 0.2588282D+01, 0.2539742D+01, 0.2492922D+01, 0.2447721D+01, &
0.2404044D+01, 0.2361806D+01, 0.2320927D+01, 0.2281333D+01, 0.2242956D+01, &
0.2205734D+01, 0.2169606D+01, 0.2134520D+01, 0.2100425D+01, 0.2067272D+01, &
0.2035019D+01, 0.2003623D+01, 0.1973047D+01, 0.1943254D+01, 0.1914210D+01, &
0.1885884D+01, 0.1858245D+01, 0.1831266D+01, 0.1804921D+01, 0.1779183D+01, &
0.1754030D+01, 0.1729439D+01, 0.1705389D+01, 0.1681861D+01, 0.1658835D+01, &
0.1636293D+01, 0.1614220D+01, 0.1592597D+01, 0.1571412D+01, 0.1550647D+01, &
0.1530291D+01, 0.1510329D+01, 0.1490750D+01, 0.1471540D+01, 0.1452690D+01, &
0.1434187D+01, 0.1416021D+01, 0.1398183D+01, 0.1380663D+01, 0.1363452D+01, &
0.1346540D+01, 0.1329921D+01, 0.1313585D+01, 0.1297525D+01, 0.1281733D+01, &
0.1266203D+01, 0.1250927D+01, 0.1235900D+01, 0.1221114D+01, 0.1206564D+01, &
0.1192244D+01, 0.1178148D+01, 0.1164271D+01, 0.1150607D+01, 0.1137152D+01, &
0.1123901D+01, 0.1110849D+01, 0.1097991D+01, 0.1085324D+01, 0.1072842D+01, &
0.1060542D+01, 0.1048420D+01, 0.1036471D+01, 0.1024694D+01, 0.1013082D+01, &
0.1001635D+01, 0.9903465D+00, 0.9792152D+00, 0.9682374D+00, 0.9574099D+00, &
0.9467299D+00, 0.9361945D+00, 0.9258008D+00, 0.9155462D+00, 0.9054280D+00, &
0.8954436D+00, 0.8855906D+00, 0.8758665D+00, 0.8662689D+00, 0.8567956D+00, &
0.8474444D+00, 0.8382130D+00, 0.8290994D+00, 0.8201015D+00, 0.8112173D+00, &
0.8024448D+00, 0.7937821D+00, 0.7852274D+00, 0.7767789D+00, 0.7684348D+00, &
0.7601934D+00, 0.7520529D+00, 0.7440118D+00, 0.7360685D+00, 0.7282214D+00, &
0.7204689D+00, 0.7128097D+00, 0.7052421D+00, 0.6977648D+00, 0.6903765D+00, &
0.6830757D+00, 0.6758611D+00, 0.6687314D+00, 0.6616853D+00, 0.6547217D+00, &
0.6478392D+00, 0.6410367D+00, 0.6343130D+00, 0.6276670D+00, 0.6210976D+00, &
0.6146036D+00, 0.6081839D+00, 0.6018376D+00, 0.5955636D+00, 0.5893609D+00, &
0.5832285D+00, 0.5771653D+00, 0.5711706D+00, 0.5652432D+00, 0.5593824D+00, &
0.5535872D+00, 0.5478567D+00, 0.5421901D+00, 0.5365864D+00, 0.5310450D+00, &
0.5255649D+00, 0.5201453D+00, 0.5147855D+00, 0.5094847D+00, 0.5042421D+00, &
0.4990569D+00, 0.4939285D+00, 0.4888560D+00, 0.4838389D+00, 0.4788763D+00, &
0.4739677D+00, 0.4691122D+00, 0.4643093D+00, 0.4595584D+00, 0.4548586D+00, &
0.4502095D+00, 0.4456105D+00, 0.4410607D+00, 0.4365598D+00, 0.4321071D+00, &
0.4277020D+00, 0.4233439D+00, 0.4190322D+00, 0.4147665D+00, 0.4105461D+00, &
0.4063706D+00, 0.4022393D+00, 0.3981518D+00, 0.3941076D+00, 0.3901060D+00, &
0.3861467D+00, 0.3822292D+00, 0.3783529D+00, 0.3745173D+00, 0.3707221D+00, &
0.3669667D+00, 0.3632507D+00, 0.3595736D+00, 0.3559349D+00, 0.3523343D+00, &
0.3487713D+00, 0.3452455D+00, 0.3417564D+00, 0.3383037D+00, 0.3348868D+00, &
0.3315055D+00, 0.3281594D+00, 0.3248479D+00, 0.3215708D+00, 0.3183276D+00, &
0.3151180D+00, 0.3119416D+00, 0.3087980D+00, 0.3056869D+00, 0.3026079D+00, &
0.2995607D+00, 0.2965448D+00, 0.2935601D+00, 0.2906060D+00, 0.2876823D+00, &
0.2847887D+00, 0.2819248D+00, 0.2790903D+00, 0.2762849D+00, 0.2735082D+00, &
0.2707600D+00, 0.2680400D+00, 0.2653477D+00, 0.2626831D+00, 0.2600456D+00, &
0.2574351D+00, 0.2548513D+00, 0.2522938D+00, 0.2497625D+00, 0.2472569D+00, &
0.2447769D+00, 0.2423222D+00, 0.2398924D+00, 0.2374874D+00, 0.2351068D+00, &
0.2327505D+00, 0.2304181D+00, 0.2281094D+00, 0.2258242D+00, 0.2235621D+00, &
0.2213230D+00, 0.2191067D+00, 0.2169128D+00, 0.2147411D+00, 0.2125914D+00, &
0.2104636D+00, 0.2083572D+00, 0.2062722D+00, 0.2042083D+00, 0.2021652D+00, &
0.2001429D+00, 0.1981409D+00, 0.1961592D+00, 0.1941975D+00, 0.1922557D+00, &
0.1903334D+00, 0.1884305D+00, 0.1865469D+00, 0.1846822D+00, 0.1828364D+00, &
0.1810091D+00, 0.1792003D+00, 0.1774097D+00, 0.1756371D+00, 0.1738824D+00, &
0.1721454D+00, 0.1704259D+00, 0.1687236D+00, 0.1670385D+00, 0.1653704D+00, &
0.1637190D+00, 0.1620842D+00, 0.1604659D+00, 0.1588638D+00, 0.1572779D+00, &
0.1557078D+00, 0.1541536D+00, 0.1526150D+00, 0.1510918D+00, 0.1495839D+00, &
0.1480911D+00, 0.1466133D+00, 0.1451504D+00, 0.1437021D+00, 0.1422684D+00, &
0.1408490D+00, 0.1394439D+00, 0.1380529D+00, 0.1366758D+00, 0.1353125D+00, &
0.1339629D+00, 0.1326268D+00, 0.1313041D+00, 0.1299946D+00, 0.1286983D+00, &
0.1274149D+00, 0.1261444D+00, 0.1248866D+00, 0.1236414D+00, 0.1224087D+00, &
0.1211883D+00, 0.1199802D+00, 0.1187841D+00, 0.1176000D+00, 0.1164277D+00, &
0.1152672D+00, 0.1141182D+00, 0.1129808D+00, 0.1118547D+00, 0.1107399D+00, &
0.1096363D+00, 0.1085437D+00, 0.1074620D+00, 0.1063911D+00, 0.1053309D+00, &
0.1042813D+00, 0.1032422D+00, 0.1022135D+00, 0.1011950D+00, 0.1001868D+00, &
0.9918860D-01, 0.9820038D-01, 0.9722203D-01, 0.9625345D-01, 0.9529455D-01, &
0.9434522D-01, 0.9340537D-01, 0.9247491D-01, 0.9155374D-01, 0.9064176D-01, &
0.8973888D-01, 0.8884502D-01, 0.8796008D-01, 0.8708398D-01, 0.8621661D-01, &
0.8535791D-01, 0.8450777D-01, 0.8366611D-01, 0.8283286D-01, 0.8200791D-01, &
0.8119120D-01, 0.8038263D-01, 0.7958213D-01, 0.7878962D-01, 0.7800501D-01, &
0.7722823D-01, 0.7645919D-01, 0.7569783D-01, 0.7494405D-01, 0.7419780D-01, &
0.7345898D-01, 0.7272754D-01, 0.7200338D-01, 0.7128645D-01, 0.7057666D-01, &
0.6987395D-01, 0.6917825D-01, 0.6848948D-01, 0.6780758D-01, 0.6713248D-01, &
0.6646410D-01, 0.6580239D-01, 0.6514727D-01, 0.6449868D-01, 0.6385656D-01, &
0.6322084D-01, 0.6259145D-01, 0.6196833D-01, 0.6135143D-01, 0.6074067D-01, &
0.6013600D-01, 0.5953735D-01, 0.5894467D-01, 0.5835789D-01, 0.5777696D-01, &
0.5720182D-01, 0.5663240D-01, 0.5606867D-01, 0.5551054D-01, 0.5495798D-01/
END
