!-----------------------------------------------------------------------
!  Interface for ionospheric electron fluid solver
!
!   Input:  dt        - time step (s)
!           n         - number of grids
!           curt(1:n) - current density at current time step (A/m^2)
!
!   Output  Te(1:n)   - isotropic electron temperature = (tpr_e+2*tpp_e)/3
!   Note: parallel & perpendicular temperatures tpr_e & tpp_e are stored
!         in common storage specified in module GSKPARM
!
!  Author: Jiannan Tu
!  Date:   Feb. 12, 2003. Modified on 7/19/2003, 11/19/2010
!------------------------------------------------------------------------
SUBROUTINE ele_fluid(dt,n,curt,Te)
    USE GSKPARM

    INTEGER :: n
    real(8),intent(in), dimension(n) :: curt
    real(8),intent(out),dimension(n) :: Te
    real(8),dimension(n) :: sgm
    real(8),dimension(n-1) :: FaieO,FaieH,FaieHe
    real(8),dimension(n-1) :: muee,mueO,mueH,mueHe,nue,Faie
    real(8),dimension(n-1) :: Pepar,Peper,Qepar,Qeper,C1,C2,tmp
    real(8) :: dt,Re,kb
    integer :: i

! Re in m; kb in J/K
    DATA Re/6371.2d3/,kb/1.38065d-23/

    do i=1,n-1
        Pepar(i)=0.5*(area(i)+area(i+1))*den_e(i)*tpr_e(i)
        Peper(i)=0.5*(area(i)+area(i+1))*den_e(i)*tpp_e(i)
    enddo

     tmp=Pepar



! arec length in meters
    sgm=sg*Re


! collision frequencies & equivelent heat flows at previous time step
    call ele_coll(n-1,FaieO,FaieH,FaieHe,muee,mueO,mueH,mueHe)
    call ht_flow(n,sgm,muee,Qepar,Qeper)


  
    open(202,file='PQ2.dat',status='unknown')
      write(202,*) '#num,mlat,tpr_e,tpp_e,Pepar,Peper,Qepar,Qeper'
      do i=1,num
         write(202,*) i,(GLL(i)+GLL(i+1))*57.3/2,tpr_e(i),tpp_e(i),&
                     Pepar(i),Peper(i),Qepar(i),Qeper(i)
      enddo
    close(202)

! change of Pepar & Peper due to collisions
    nue=mueO+mueH+mueHe

!*************Faie, C1 and C2 have been changed*************
!    Faie=FaieO+FaieH+FaieHe
     Faie=FaieO*mueO+FaieH*mueH+FaieHe*mueHe
!    C1=(Faie-0.4*nue)*dt
!    C2=(2.0*Faie+0.4*nue)*dt
     C1=Faie-0.4*nue
     C2=2.0*Faie+0.4*nue
!************************************************************

! update electron density & velocity
!      CALL ele_den(den_O,den_H,vel_e,area,rrm,dt,n,den_e)
    CALL ele_den(n)
    CALL ele_vel(n,curt)

    open(201,file='PQ1.dat', status='unknown')
    do i=1,num
        write(201,*) i,(GLL(i)+GLL(i+1))*57.3/2,den_e(i),area(i),vel_e(i)
    enddo
    close(201)


! calculate electron parallel temperature

    CALL par_temp(n,sgm,dt,Qepar,Qeper,Pepar,Peper,C1,C2)

! calculate electron perpendicular temperature
   
    CALL per_temp(n,sgm,dt,Qeper,tmp,Peper,C1,C2)

! update collision frequencies
    call ele_coll(n-1,FaieO,FaieH,FaieHe,muee,mueO,mueH,mueHe)

! update equivelent heat flows
    call ht_flow(n,sgm,muee,Qepar,Qeper)

    open(203,file='PQ3.dat',status='unknown')
      write(203,*) '#num,mlat,tpr_e,tpp_e,Pepar,Peper,Qepar,Qeper'
      do i=1,num
         write(203,*) i,(GLL(i)+GLL(i+1))*57.3/2,tpr_e(i),tpp_e(i),&
                     Pepar(i),Peper(i),Qepar(i),Qeper(i)
      enddo
    close(203)

    Te(1:n)=(tpr_e(1:n)+2.0d0*tpp_e(1:n))/3.0d0

! qpr_e=kb*Qepar/3/area; qpp_e=2kb*Qeper/area. Note Qepar & Qeper are calculated
! with den_e in cm^-3 so there is a factor of 10^6
! electron heat flow in J m^-2 s^-1
  !  qpr_e(1:n)=3.0d6*kb*Qeper(1:n)/area(1:n)
  !  qpp_e(1:n)=2.0d6*kb*Qeper(1:n)/area(1:n)

!*********************************************************
!    qpr_e(1:n)=5.0d8*kb*Qepar(1:n)/area(1:n)
!    qpp_e(1:n)=1.0d9*kb*Qeper(1:n)/area(1:n)
!*********************************************************

end subroutine
!-----------------------------------------------------------------------
!  Solves for ionospheric electron density at current time step.
!
!    Input:  den_O  - iono O+ density at current time step (cm^-3)
!            den_H  - iono H+ density at current time step (cm^-3)
!            vel_e  - electron velocity at previous time step (m/s)
!            area   - flux tube cell area (cm^2)
!            rrm    - geocentrical distance of grids (in meter)
!            dt     - time step (s)
!            n      - actual number of grids
!            den_e  - electron densities at previous time step (cm^-3)
!
!    Output: den_e  - electron density at current time step (cm^-3)
!
!  Author: Jiannan Tu
!  Date:   Feb. 7, 2003.
!-----------------------------------------------------------------------
!SUBROUTINE ele_den(den_O,den_H,vel_e,area,rrm,dt,n,den_e)
SUBROUTINE ele_den(n)
    USE GSKPARM

    INTEGER :: n,j
!      REAL(8) area(nc),rrm(nc),dt,tmp,a(n),b(n),c(n),r(n),AN(n)

! coefficients of tridiagnoal linear set
!      b(1)=1.
!      c(1)=0.
!      r(1)=area(1)*(den_H(1)+den_O(1))
!      a(n)=0.
!      b(n)=1.
!      r(n)=area(n)*(den_H(n)+den_O(n))
!      do j=2,n-1
!         a(j)=-vel_e(j)*dt
!         tmp=rrm(j+1)-rrm(j-1)
!         b(j)=tmp+(vel_e(j+1)-vel_e(j-1))*dt
!         c(j)=-a(j)
!         r(j)=tmp*area(j)*den_e(j)
!      enddo

!  call tridiag solver to get A(I)*ne(I) (I=1,2,...,n)
!      CALL tridiag(a,b,c,r,AN,n)

!  new electron density
    do j=1,n
!         den_e(j)=AN(j)/area(j)
       den_e(j)=den_O(j)+den_H(j)+den_He(j)
    enddo


end subroutine
!---------------------------------------------------------------------
!  Solve ionospheric electron parallel velocity at current time step
!
!    Input:  n      - number of grid points
!            curt   - parallel current density at current time (A/m^2)
!
!    Output: none
!    Note: calculated parallel electron velocities are stored in common
!          storage specified in mopdule GSKPARM
!
!  Author: Jiannan Tu
!  Date:   Feb 11, 2003
!-------------------------------------------------------------------
SUBROUTINE ele_vel(n,curt)
    USE GSKPARM

    INTEGER, intent(in) :: n
!********curt needs to be current density (A/m^2)
    real(8), intent(in), dimension(n) :: curt   
    real(8) :: q
    integer :: j

    data q/1.6022d-19/

    do j=1,n
      if(den_e(j).gt.0.0d0) then
        vel_e(j)=(den_O(j)*vel_O(j)+den_H(j)*vel_H(j)+den_He(j)*vel_He(j)- &
                  curt(j)/q)/den_e(j)
      else
        vel_e(j)=0.0d0
      endif
    enddo

!**************************************
    call smoothp(vel_e,n-1)
!**************************************

end subroutine
!--------------------------------------------------------------------------
!  Solve for parallel electron ionospheric temperatures at current time step
!
!    Input:   n           - number of grids
!             sqm(1:n)    - arc distance of the grids (m)
!             dt          - time step (s)
!             Qepar(1:n)  - parallel electron heat flx in SI unit
!             Qeper(1:n)  - perpendicular electron heat flx in SI unit
!             Pepar(1:)   - parallel Pepar in SI units
!             dPepar(1:n) - change of Pepar due to collisions
!
!    Output:  none
!
!    Note: Caculated parallel Te// are in common storage in module GSKPARM
!
!  Author: Jiannan Tu
!  Date:   Feb. 10, 2003. Modified on July 19, 2003; Nov 19, 2010
!--------------------------------------------------------------------------
SUBROUTINE par_temp(n,sgm,dt,Qepar,Qeper,Pepar,Peper,C1,C2)
    use inparm
    USE GSKPARM

    INTEGER n,j

    REAL(8),dimension(n) :: sgm
    REAL(8),dimension(n-1) :: Qepar,Qeper,Pepar,Peper,C1,C2
    REAL(8),dimension(n-1) :: a,b,c,r,aa,bb,cc,x,y
    REAL(8) :: dt,alfa,temp,temp2,temp3
    SAVE

    a=0.0
    b=0.0
    c=0.0
    r=0.0

    aa=0.0
    bb=0.0
    cc=0.0

    
    do j=2,n-2
       temp2=0.5*(vel_e(j-1)+vel_e(j))
       temp3=0.5*(vel_e(j+1)+vel_e(j))
       
       if (temp2.gt.0.0d0) then 
           aa(j)=-1*temp2
       endif

       if (temp3.gt.0.0d0.and.temp2.gt.0.0d0) then
          bb(j)=temp3
         else if (temp3.lt.0.0d0.and.temp2.lt.0.0d0) then
          bb(j)=-1*temp2
         else if (temp3.gt.0.0d0.and.temp2.lt.0.0d0) then
          bb(j)=temp3-temp2
       else
          bb(j)=0.0
       endif

       if (temp3.lt.0) then 
           cc(j)=temp3
       endif

    enddo
          
 

    b(1)=1.0d0
    c(1)=0.0d0
    r(1)=0.5*(area(1)+area(2))*den_e(1)*FT_e(1)
!    r(1)=area(1)*den_e(1)*tpr_e(1)

      
    do j=2,n-2

            alfa=2*dt/(sgm(j+1)-sgm(j)+sgm(j+2)-sgm(j-1))

            a(j)=2*alfa*aa(j)
            temp=alfa*(vel_e(j+1)-vel_e(j-1))-C1(j)*dt
            b(j)=1.0d0+temp+2*alfa*bb(j)
            c(j)=2*alfa*cc(j)
            r(j)=(1.0-temp)*Pepar(j)-alfa*(Qepar(j+1)-Qepar(j-1))+ &
                  2.0d0*dt*(Qeper(j)*dA(j)+C2(j)*Peper(j))
        
    enddo

    a(n-1)=0.0d0
    b(n-1)=1.0d0
    r(n-1)=0.5*(area(n)+area(n-1))*den_e(n-1)*FT_e(2)
!    r(n)=area(n)*den_e(n)*tpr_e(n)


    call tridiag(a,b,c,r,Pepar,n-1)

    do j=2,n-2
        x(j)=a(j)*Pepar(j-1)+b(j)*Pepar(j)+c(j)*Pepar(j+1)
        y(j)=r(j)
    enddo
    x(1)=r(1)
    y(1)=r(1)
    x(n-1)=r(n-1)
    y(n-1)=r(n-1)

    open(101,file='test_solution1.dat',status='unknown')
    write(101,*) '#num,mlat,left side, right side'
    do j=1,n-1
        write(101,*) j,x(j),y(j)
    enddo
    close(101)
! update parallel temperature
    do j=1,n-1
        if(den_e(j) .gt. 0.0d0) then
            tpr_e(j)=abs(Pepar(j))/(0.5*(area(j)+area(j+1))*den_e(j))
        else
            tpr_e(j)=0.0d0
        endif
    enddo
    tpr_e(n)=0.0d0

!**************************************
    call smoothp(tpr_e,n-1)
!**************************************
! check if there is any neagative parallel temperature
    do j=2,n-1
        if(tpr_e(j).lt.0.0) then
            write(63,'(a,i3,f11.2)') 'at element j= ',j,tpr_e(j)
            write(63,'(a)') 'Shutdown in ELE_FLUID subroutine par_temp'
            stop 'Shutdown: Negative tpr_e element'
        endif
    enddo
end subroutine
!-------------------------------------------------------------------------
!  Solve for perpendicular ionospheric electron temperatures at current
!  time step
!
!    Input:   n           - number of grids
!             sqm(1:n)    - arc distance of the grids (m)
!             dt          - time step (s)
!             Qeper(1:n)  - perpendicular electron heat flx in SI unit
!             Peper(1:n)  - perpendicular Peper in SI unit
!             dPeper(1:n) - change of Peper due to collisions
!
!    Output:  none
!
!    Note: Caculated perpendicular Te| are in common storage in module GSKPARM
!
!  Author: Jiannan Tu
!  Date:   Feb. 10, 2003. Modified on July 19, 2003; Nov 19, 2010
!-------------------------------------------------------------------------
SUBROUTINE per_temp(n,sgm,dt,Qeper,Pepar,Peper,C1,C2)
    use inparm
    USE GSKPARM

    INTEGER n,j

    REAL(8),dimension(n) :: sgm
    REAL(8),dimension(n-1) :: Qeper,Pepar,Peper,C1,C2
    REAL(8),dimension(n-1) :: a,b,c,r,aa,bb,cc,x,y
    REAL(8) :: dt,alfa,temp,temp2,temp3
    SAVE

    a=0.0
    b=0.0
    c=0.0
    r=0.0

    aa=0.0
    bb=0.0
    cc=0.0
    
    do j=2,n-2
       temp2=0.5*(vel_e(j-1)+vel_e(j))
       temp3=0.5*(vel_e(j+1)+vel_e(j))
       
       if (temp2.gt.0.0d0) then 
           aa(j)=-1*temp2
       endif

       if (temp3.gt.0.0d0.and.temp2.gt.0.0d0) then
          bb(j)=temp3
         else if (temp3.lt.0.0d0.and.temp2.lt.0.0d0) then
          bb(j)=-1*temp2
         else if (temp3.gt.0.0d0.and.temp2.lt.0.0d0) then
          bb(j)=temp3-temp2
       else
          bb(j)=0.0
       endif

       if (temp3.lt.0) then
           cc(j)=temp3
       endif

    enddo
          

    b(1)=1.0d0
    c(1)=0.0d0
    r(1)=0.5*(area(1)+area(2))*den_e(1)*FT_e(1)
!    r(1)=area(1)*den_e(1)*tpp_e(1)
     
    do j=2,n-2

            alfa=2*dt/(sgm(j+1)-sgm(j)+sgm(j+2)-sgm(j-1))

            a(j)=2*alfa*aa(j)
            temp=0.5*dt*(vel_e(j)*dA(j)+C2(j))
            b(j)=1.0d0+2*alfa*bb(j)+temp
            c(j)=2*alfa*cc(j)
            r(j)=(1.0-temp)*Peper(j)-alfa*(Qeper(j+1)-Qeper(j-1))-&
                  dt*(Qeper(j)*dA(j)+C1(j)*Pepar(j))
      
    enddo

    a(n-1)=0.0d0
    b(n-1)=1.0d0
    r(n-1)=0.5*(area(n)+area(n-1))*den_e(n-1)*FT_e(2)
!    r(n)=area(n)*den_e(n)*tpp_e(n)
 
    call tridiag(a,b,c,r,Peper,n-1)

    do j=2,n-2
        x(j)=a(j)*Peper(j-1)+b(j)*Peper(j)+c(j)*Peper(j+1)
        y(j)=r(j)
    enddo
    x(1)=r(1)
    x(n-1)=r(n-1)
    y(1)=r(1)
    y(n-1)=r(n-1)

    open(101,file='test_solution2.dat',status='unknown')
    write(101,*) '#num,mlat,left side, right side'

    do j=1,n-1
        write(101,*) j,x(j),y(j)
    enddo
    close(101)
 
    do j=1,n-1
        if(den_e(j) .gt. 0.0d0) then
            tpp_e(j)=abs(Peper(j))/(0.5*(area(j)+area(j+1))*den_e(j))
        else
            tpp_e(j)=0.0d0
        endif
    enddo
!**************************************
    call smoothp(tpp_e,n-1)
!**************************************

    do j=2,n-1
        if(tpp_e(j).lt.0.0) then
            write(63,'(a,i3,f11.2)') 'at element j= ',j,tpp_e(j)
            write(63,'(a)') 'Shutdown in ELE_FLUID subroutine per_temp'
            stop 'Shutdown: Negative tpp_e element'
        endif
    enddo
end subroutine
!-------------------------------------------------------------------
!  Calculate momentum transfer collision frequency between e and e,
!  e and O+, e and H+, e and He+.
!
!    Input:  n - number of grids
!    Output: FaieO - Factor FAIeO
!            FaieH - Factor FAIeH
!            muee  - electron - electron collision frequency (s^-1)
!            mueO  - electron - O+ ion collision frequency (s^-1)
!            mueH  - electron - H+ ion collision frequency (s^-1)
!
!  Author: Jiannan Tu
!  Date:   Feb 9, 2003. Modified 3/11/10
!-------------------------------------------------------------------
SUBROUTINE ele_coll(n,FaieO,FaieH,FaieHe,muee,mueO,mueH,mueHe)
    USE GSKPARM

    INTEGER, intent(in) :: n

    REAL(8),intent(out),dimension(n) :: FaieO,FaieH,FaieHe,muee,mueO,mueH,mueHe

    REAL(8) :: kbme,alpha1,xeO2,xeH2,xeHe2,Te,FAI,DEX,Te15
    INTEGER :: j

! here value of kb/me  in SI unit
    DATA kbme/1.515635d7/


    muee=0.0d0
    mueO=0.0d0
    mueH=0.0d0
    mueHe=0.0d0
    FaieO=0.0d0
    FaieH=0.0d0
    FaieHe=0.0d0

    do j=1,n
      Te=(tpr_e(j)+2.*tpp_e(j))/3.

      if(den_e(j).le.0.0d0.or.Te.le.0.0d0) cycle

      Te15=Te**1.5d0

      ! e - e momentum transfer collision frequency	(s^-1) (Ganguli et al.,
      ! PSS, 35(6),703-713, 1987)
      muee(j)=27.191451d0*den_e(j)/Te15

      ! alpha1 is the square of velocity in unit of m/s
      alpha1=2.0d0*kbme*Te  !(=2kb(Te/me+To/mo) appr = 2kbTe/me)
      xeO2=(vel_e(j)-vel_O(j))**2/alpha1
      FaieO(j)=FAI(xeO2)

      ! e - O+ momentum transfer collision frequency (s^-1)
      if(xeO2.gt.80.0d0) then
        dex=0.0d0
      else
        dex=exp(-xeO2)
      endif
      mueO(j)=38.45452d0*den_O(j)/Te15*dex

      ! alpha1=2.0*kbme*Te  !(=2kb(Te/me+Th/mH) appr = 2kbTe/me)
      xeH2=(vel_e(j)-vel_H(j))**2/alpha1
      FaieH(j)=FAI(xeH2)

      ! e - H+ momentum transfer collision frequency (s^-1)
      if(xeH2.gt.80.0d0) then
        dex=0.0d0
      else
        dex=exp(-xeH2)
      endif
      mueH(j)=38.45452d0*den_H(j)/Te15*dex

      ! alpha1=2.0*kbme*Te  !(=2kb(Te/me+The/mHe) appr = 2kbTe/me)
      xeHe2=(vel_e(j)-vel_He(j))**2/alpha1
      FaieHe(j)=FAI(xeHe2)

      ! e - He+ momentum transfer collision frequency (s^-1)
      if(xeHe2.gt.80.0d0) then
        dex=0.0d0
      else
        dex=exp(-xeHe2)
      endif
      mueHe(j)=38.45452d0*den_He(j)/Te15*dex
    enddo

end subroutine
!---------------------------------------------
!  Calculate value of FAIeO, FAIeH
!
!  Author: Jiannan Tu
!  Date:   Feb 9, 2003
!---------------------------------------------
REAL(8) function FAI(x)

    real(8), intent(in) :: x

    FAI=2.0d0/5.0d0*x+4.0d0/35.0d0*x*x+8.0d0/315.0d0*x*x*x

end function
!---------------------------------------------------------------------
!  Calculate electron heat flow equivelent Qe (parallel and perpendicular)
!
!    Input:   n          - number of grids
!             sgm(1:n)   - arc length of grids (m)
!             muee(1:n)  - electron - electron collision frequency (s^-1)
!
!    Output:  Qepar(1:n) - parallel electron heat flow (normalized by kb/me)
!             Qeper(1:n) - perpendicular ele heat flow (normalized by kb/me)
!
!  Author: Jiannan Tu
!  Date:   Feb. 10, 2003; modified Nov 19, 2010
!---------------------------------------------------------------------
SUBROUTINE ht_flow(n,sgm,muee,Qepar,Qeper)
    USE GSKPARM

    INTEGER, intent(in) :: n

    REAL(8), intent(in), dimension(n) :: sgm
    REAL(8), intent(in), dimension(n-1) :: muee
    reAL(8), intent(out), dimension(n-1) :: Qeper,Qepar
    REAL(8) ds,dTepar(n-1),dTeper(n-1),kbme,Le(n-1),vth,tmp(n-1)
    INTEGER :: j
    real(8) kb

! value of kb/me in SI unit, kb in SI unit
    data kbme/1.5156322d7/ , kb/1.38065d-23/

    Qepar=0.0
    Qeper=0.0

    do j=1,n-1
        if(den_e(j) .le. 0.0 .or. muee(j) .le. 0.0 .or. &
         tpr_e(j) .le. 0.0 .or. tpp_e(j) .lt. 0.0) cycle

        !parallel thermal velocity in m/s
        vth=sqrt(tpr_e(j)*kbme)

        if(j ==1) then
            ! arc length in m
            ds=(sgm(j+2)-sgm(j))*0.5+dSL(1)

            dTepar(j)=(tpr_e(j+1)-tpr_e(j)+dFT_e(1))/ds
            dTeper(j)=(tpp_e(j+1)-tpp_e(j)+dFT_e(1))/ds

            Le(j)=min(1.25d0*vth/muee(j),(sgm(j+2)-sgm(j))*0.5)
        else if(j == n-1) then
            ds=(sgm(j+1)-sgm(j-1))*0.5+dSL(2)

            dTepar(j)=(tpr_e(j)-tpr_e(j-1)+dFT_e(2))/ds
            dTeper(j)=(tpp_e(j)-tpp_e(j-1)+dFT_e(2))/ds

            Le(j)=min(1.25d0*vth/muee(j),(sgm(j+1)-sgm(j-1))*0.5)
        else
            ds=0.5*(sgm(j+2)+sgm(j+1)-sgm(j)-sgm(j-1))

            dTepar(j)=(tpr_e(j+1)-tpr_e(j-1))/ds
            dTeper(j)=(tpp_e(j+1)-tpp_e(j-1))/ds

            Le(j)=min(1.25d0*vth/muee(j),sgm(j+1)-sgm(j))
        endif

        ! calculate Qepar and Qeper (area in m^2; vth in m/s; den_e in cm^-3)
        tmp(j)=-Le(j)*0.5*(area(j)+area(j+1))*den_e(j)*vel_e(j)

        Qepar(j)=3.0d0*tmp(j)*dTepar(j)
        Qeper(j)=tmp(j)*(dTeper(j)+(1.0-tpp_e(j)/tpr_e(j))*tpp_e(j)*dA(j))
    enddo


!    Qepar(1)=2.0d-9*area(1)*(FheatE(1)/kb)
!    Qeper(1)=1.0d-9*area(1)*(FheatE(1)/kb)*2
!    Qepar(n)=2.0d-9*area(n)*(FheatE(2)/kb)
!    Qeper(n)=1.0d-9*area(n)*(FheatE(2)/kb)*2

!    call smoothp(Qepar,n)
!    call smoothp(Qeper,n)

    open(205,file='Q2.dat',status='unknown')
      write(205,*) '#num,mlat,dTepar,dTeper,vel_e,Le,den_e,A,tmp,Qepar,Qeper'
      do j=1,num
         write(205,*) j,(GLL(j)+GLL(j+1))*57.3/2,dTepar(j),dTeper(j),vel_e(j),&
                      Le(j),den_e(j),area(j),tmp(j),Qepar(j),Qeper(j)
      enddo
    close(205)
    

! equivelent heat flow at the southern and northern BC
! heat flow at northern end
!---------------------
!  ehtf - iostropic electron heat flow in erg/(cm^2 s) from FLIP. 
!  Qeper = 2/5*ehtf/kb & Qepar = 6/5*ehtf/kb as here defined Qeper & Qepar
!  Note den_e is in cm^-3 so that there is a factor of 10^-6 in Qeper & Qepar
!---------------------
!    Qeper(1)=0.4d-9*area(1)*(ehtf(1)/kb)
!    Qepar(1)=3.0d0*Qeper(1)

! heat flow at southern end
!    Qeper(n)=0.4d-9*area(n)*(ehtf(2)/kb)
!    Qepar(n)=3.0d0*Qeper(n)

end subroutine
!-----------------------------------------------------------------------
!   Solves for a vector u(1:n) of length n the tridiagonal linear
!   set given by a tridigonal equation. a(1:n), b(1:n), c(1:n),
!   and r(1:n) are input vectors and are not modified.
!   Parameter NMAX is the maximum expected value of n.
!
!   Press, W. H., et al., Numerical Recipes in Fortran vol.1,Chap 2.4,
!      1995.
!   On-line books: http://www.library.cornell.edu/nr/bookfpdf.html 
!-----------------------------------------------------------------------
SUBROUTINE tridiag(a,b,c,r,u,n)
    INTEGER n,NMAX
    REAL(8) a(n),b(n),c(n),r(n),u(n)
    PARAMETER (NMAX=500)
    INTEGER j
    REAL(8) bet,gam(NMAX)  ! One vector workspace, gam is needed 

    if(b(1).eq.0.0d0) stop 'tridiag: rewite equations'
    ! if this happens then you should rewite your equations as a set of
    ! order of N -1, with u2 trivally eliminated.

    bet=b(1)
    u(1)=r(1)/bet
    do j=2,n                    ! Decomposition and forward substitution
      gam(j)=c(j-1)/bet
      bet=b(j)-a(j)*gam(j)
      if(bet.eq.0.0d0) then     ! Algorithm failed
        write(63,'(a)') 'tridiag failed'
        stop
      endif
      u(j)=(r(j)-a(j)*u(j-1))/bet
    enddo
    do j=n-1,1,-1                ! Backsubstitution
      u(j)=u(j)-gam(j+1)*u(j+1)
    enddo

end subroutine
!--------------------------------------------------------------------------
!  Calculate terms related to collisions in parallel & perpendicular
!  temperature equations
!
!  Author: Jiannan Tu
!  Date:   November 6, 2010
!--------------------------------------------------------------------------
subroutine dPepardPeper(n,FaieO,FaieH,FaieHe,mueO,mueH,mueHe,dPepar,dPeper)
    USE gskparm

    INTEGER, intent(in) :: n
    real(8), intent(in), dimension(n) :: FaieO,FaieH,FaieHe,mueO,mueH,mueHe
    real(8), intent(out), dimension(n) :: dPepar,dPeper
    real(8), dimension(n) :: mue,muFaie,Te

    dPepar=0.0
    dPeper=0.0

    mue=mueO+mueH+mueHe
    muFaie=mueO*FaieO+mueH*FaieH+mueHe*FaieHe
    Te=(tpr_e(1:n)+2.0*tpp_e(1:n))/3.0

    dPepar=area*den_e*mue*(0.8*(tpp_e(1:n)-tpr_e(1:n))+6.0*Te*muFaie)
    dPeper=-0.5*dPepar
end

