!**********************************************************************
!**********************************************************************
!  Rountine to calculate q-coordinate along the field line
!  
!  Input:   num - number of grids on the field line
!           Req - L-shell or equatorial radial distance of field line
!           Z   - array of altitude of the grids, from FLIP routine
!           scalg - scale factor in distributing grids
!           jgn - grid index of GSK lower boundary in FLIP zone
!
!  Output:  qval - array of q values of the grids on the field line
!
!  Written by J Tu 1/27/10
!**********************************************************************
subroutine qvalue(num,Req,Z,scalg,jbn,qval)
    integer num,jbn,j,pts

    real(8) Req,scalg,qval(num+1),rr,GLL,scalk,scx,Re,qmax,dx
    real(8) Z(401)

    data Re/6371.2d3/ !Re in m

    rr=1.0d0+1.0d3*Z(jbn)/Re
    GLL=acos(sqrt(rr/Req))   ! GLL latitude
    qmax=sin(GLL)/rr**2

    pts=num/2+1
    SCALK=1.0/SINH(scalg*qmax)

    qval(num/2+1)=0.0d0
    do j = 1, num/2
        DX=1.0d0-float((J-1))/float((PTS-1))
!...    qval=the dipole coord  determined from -- sinh(kq)=dx
        SCX=DX/SCALK
        qval(J) = LOG(SCX+SQRT(SCX**2+1.0d0))/scalg
        qval(num+2-J)=-qval(J)
    enddo

end subroutine
!**********************************************************************
!**********************************************************************
!  Rountine to calculate (r, lat) for the grids on a field line
!  
!  Input:   Req - L-shell or equatorial radial distance of field line
!
!  Written by J Tu 1/27/10 
!**********************************************************************
subroutine calc_grid(nst,Req)
    use inparm
    use gskparm

    integer nst,LNBLNK,j

    integer rank,ierr,status
    common /mpi/rank,ierr,status

    real(8) Req,d, e, y, tem, d1, por, x, sxh, volfunc
    real(8) x0,x1,x2,y0,y1,z0,z1,Re,sg0,gme,Be,A0,cc,dd

    character*6 suff

! Earth's mass Me = 5.9742e24 kg; G const = 6.6726e-11 m^3 kg^-1 s^-2
! gme=2*G*Me/Re^3
    data por/0.33333333333333d0/,Re/6371.2d3/,gme/-3.0827654d-6/
    data Be/3.11d-5/,A0/0.01d0/,cc/0.288675134595d0/,dd/1.732050808d0/

    rr(num/2+1)=Req
    GLL(num/2+1)=0.0d0
    bfd(num/2+1)=Be/Req**3

    do j=1, num/2
      d=1.d0/(Req*qval(j)*qval(j))
      e=-1.d0/(qval(j)*qval(j))
      d1=d**2/2.d0
      tem=sqrt(d1*d1-2.37037037037d0*e*e*e)
      y=(d1+tem)**por-(tem-d1)**por

      ! rr - radial distance in unit of Re
      ! GLL - magnetic latitude (in rad)
      rr(j)=(sqrt(2.d0*sqrt(y*y-4.d0*e)-y)-sqrt(y))/2.d0
      GLL(j)=asin(qval(j)*rr(j)*rr(j))

      x=sin(GLL(j))
      sxh=sqrt(1.d0+3.d0*x*x)

      rr(num+2-j)=rr(j)
      GLL(num+2-j)=-GLL(j)

!     magnetic field (Tesla)
      bfd(j)=Be/rr(j)**3*sxh
      bfd(num+2-j)=bfd(j)
    enddo

    x=sin(GLL(1))
    sxh=sqrt(1.d0+3.d0*x*x)
    sg0=Req*(x/2.d0*sxh+cc*log(dd*x+sxh))

    ! arc length (Re) from N->S. sg=0 at northern boundary
    sg(1)=0.0d0
    do j=2,num+1
      x=sin(GLL(j))
      sxh=sqrt(1.d0+3.d0*x*x)

      sg(j)=sg0-Req*(x/2.d0*sxh+cc*log(dd*x+sxh))
    enddo

!---- claculate area and vol of grid cells
!------------------------------------------------------------------
!  area of at the flux tube ends (at zb) = 0.01 m^2
!  area in m^-2; vol in m^-3; alt in km; sg & rr in Re
!------------------------------------------------------------------
    x0=sin(GLL(1))
    y0=cos(GLL(1))
    z0=sqrt(1.d0+3.d0*x0*x0)

    do j=1,num/2+1
      x1=sin(GLL(j))
      x2=x1*x1
      y1=cos(GLL(j))
      z1=sqrt(1.d0+3.d0*x2)

      ! cross section area of the flux tube
      area(j)=(rr(j)/rr(1))**3*z0/z1
      area(num+2-j)=area(j)

      ! (dA/ds)/A=(dB/ds)/B (in m^-1)
      dA(j)=3.0d0*x1*(3.0d0+5.0d0*x2)/(rr(j)*z1**3)
      dA(num+2-j)=-dA(j)

      ! mirror force factor
      mirrf(j)=dA(j)/2.0
      mirrf(num+2-j)=-mirrf(j)

      ! gravitaty (gme negative; in Re/s^2)
      gm(j)=gme/(rr(j)*rr(j))*x1/z1
      gm(num+2-j)=-gm(j)

      ! factor for 2nd term of centrifugal force
      cf(j,1)=3.0d0*(1.0d0+x2)*(y1/z1)**4/rr(j)
      cf(num+2-j,1)=cf(j,1)

      ! factor for 3rd term of centrifugal force
      cf(j,2)=6.0d0*(1.0+x2)*x1*y1**6/(rr(j)*z1**5)
      cf(num+2-j,2)=-cf(j,2)
    enddo
    area(1:num+1)=area(1:num+1)*A0
    dA(1:num+1)=dA(1:num+1)/Re
    cf(1:num+1,1)=cf(1:num+1,1)*Veq*dtg
    cf(1:num+1,2)=cf(1:num+1,2)*Veq2*dtg

! grid cell volume (m^3)
    do j=1,num/2
      c_vol(j)=volfunc(GLL(j))-volfunc(GLL(j+1))
      c_vol(num-j+1)=c_vol(j)
    enddo
    c_vol(1:num)=(A0*Re*Req**4/rr(1)**3*z0/7.0d0)*c_vol(1:num)

! volume of flux tube segments centered at grid points
    do j=1,num/2+1
        if(j == 1) then
            vol(j)=0.5*c_vol(j)
        else
            vol(j)=0.5*(c_vol(j-1)+c_vol(j))
        endif
        vol(num+2-j)=vol(j)
    enddo

! altitude (km)
    do j=1,num+1
      alt(j)=(rr(j)-1.d0)
    enddo
    alt(1:num+1)=alt(1:num+1)*Re/1.0d3

! Inverse of ds (Re^-1)
    do j=1,num
      ods(j)=1.d0/(sg(j+1)-sg(j))
    enddo

    
	
	if(rank.eq.0) then
	  if(nst==1.or.(mod(nst,iout)==0.and.drift=='Yes')) then
         write(suff,'(i6)') nst
         suff=adjustl(suff)

      ! Tco, Tch, & Tche are O+, H+, & He+ gyroperiod (sec)
         open(232,file='/home/wangy/DyFk/PLOT/gskgrid-'//trim(suff)//'.dat')
         write(232,10) '#','r(Re)','mlat','alt(km)','arc len(Re)','area(cm^-2)', &
         'vol(cm^-3)','g(Re/s^2)','Bfeild(T)','mirrf','cf1','cf2','Tco','Tch','Tche'
10       format(A1,6x,A5,9x,A4,8x,A7,4x,A11,2x,A11,3x,A10,3x,A9,4x,A9,6x,A5, &
                2(9x,A3),11x,A3,9x,A3,10x,A4)
         do j=1,num+1
            write(232,12) j,rr(j),GLL(j)*57.2958,alt(j),sg(j),area(j),vol(j),gm(j), &
                   bfd(j),mirrf(j),cf(j,1),cf(j,2),1.05d-6/bfd(j),6.564d-8/bfd(j), &
                   2.6256d-7/bfd(j)
         enddo
12       format(i3,14es13.5)
         close(232)
       endif
	 endif


end subroutine
